#!/bin/bash
#
# NDI Security framework configuration script.
# Requires root privileges to execute properly.
# Must have NDI.SecClientConfig tool in the same directory.
#

# error code var
declare -i iErr=0

############ subroutines ############

eqCheckExitStatus()
{
	iErr=$?
	if [ $iErr != 0 ] ; then
		echo "Security config failed. iErr = $iErr"
		exit $iErr    
	fi
}

######################################

PrintWarningIfErrorAndContinue()
{
    iErr=$?
    if [ $iErr != 0 ] ; then
        echo "Warning: operation failed. iErr = $iErr"
    fi
}

######################################

eqAssertDirPath()
{
    if [ ! -d "$strDirPath2Assert" ] ; then
        mkdir -p "$strDirPath2Assert"
        eqCheckExitStatus
    fi
}

######################################

EnrollAllComponents()
{
    echo "sfsHost =" $sfsHost
    echo "datacenterName =" $datacenterName
    echo "printGroupsList =" $printGroupsList
    echo "enrollDrcMode =" $enrollDrcMode

    if [ $iNumParams != 6 ] && [ $iNumParams != 7 ] ; then
        echo "Invalid number of input parameters: $iNumParams. Must have 6 or 7."
        Usage
        exit 1
    fi

    echo "configuring bootstrap url..."
    "$scriptDir/NDI.SecClientConfig" configure-bootstrap-url -u "$adminUser" -p "$adminPassword" -h "$sfsHost" -dn "$datacenterName"
    eqCheckExitStatus

    echo "enrolling client component: $printAdminComponentName..."
    "$scriptDir/NDI.SecClientConfig" enroll-client -u "$adminUser" -p "$adminPassword" -nn "$printAdminComponentName" -gread "allow"
    eqCheckExitStatus

    echo "enrolling client component: $printClientComponentName..."
    "$scriptDir/NDI.SecClientConfig" enroll-client -u "$adminUser" -p "$adminPassword" -nn "$printClientComponentName" -gread "allow" -groups "$printGroupsList"
    eqCheckExitStatus

    echo "enrolling client component: $sharedEngineClientName..."
    "$scriptDir/NDI.SecClientConfig" enroll-client -u "$adminUser" -p "$adminPassword" -nn "$sharedEngineClientName"
    eqCheckExitStatus

    if [ "$enrollDrcMode" == "enroll_drc" ] ; then

        drcSystemNameConfig="/Library/Application Support/Equitrac/DRE/EQDRESYS.cfg"
        if [ ! -e "$drcSystemNameConfig" ] ; then
            echo "Error: DRC System name is not configured. Can't find '$drcSystemNameConfig'."
            echo "Please run EQPrinterUtilityX and use Preferences dialog to configure DRC System name."
            exit 1
        fi

        echo "enrolling client component: $drcClientName..."
        "$scriptDir/NDI.SecClientConfig" enroll-client -u "$adminUser" -p "$adminPassword" -nn "$drcClientName"
        eqCheckExitStatus

        thisHostName=$(<"$drcSystemNameConfig")
        drcEndpointUrl="https://$thisHostName:2938/"
        echo "enrolling service component: $drcServicetName url: $drcEndpointUrl ..."
        drcKeychainBasePath="/Library/Application Support/Equitrac"
        if [ ! -d "$drcKeychainBasePath" ] ; then
            strDirPath2Assert="$drcKeychainBasePath"
            eqAssertDirPath
            chown root:admin "$drcKeychainBasePath"
            eqCheckExitStatus
            chmod u=rwx,g=rwx,o=rx,-t "$drcKeychainBasePath"
            eqCheckExitStatus
        fi
        "$scriptDir/NDI.SecClientConfig" enroll-service -u "$adminUser" -p "$adminPassword" -owner "$drcClientName" -name "$drcServicetName" -url "$drcEndpointUrl" -namespace Equitrac -hostname "$thisHostName" -keychain "$drcKeychainBasePath"/"eqmanual.keychain"
        eqCheckExitStatus
        chown root:wheel "$drcKeychainBasePath"/"eqmanual.keychain.password"
        eqCheckExitStatus
    else
        echo "skipping DRC enrollment."
    fi
}

######################################

UnenrollAllComponents()
{
    echo "unenrollDrcMode =" $unenrollDrcMode

    if [ $iNumParams != 3 ] && [ $iNumParams != 4 ] ; then
        echo "Invalid number of input parameters: $iNumParams. Must have 3 or 4."
        Usage
        exit 1
    fi

    echo "unenrolling client component: $printAdminComponentName..."
    "$scriptDir/NDI.SecClientConfig" unenroll-client -u "$adminUser" -p "$adminPassword" -un "$printAdminComponentName"
    PrintWarningIfErrorAndContinue

    echo "unenrolling client component: $printClientComponentName..."
    "$scriptDir/NDI.SecClientConfig" unenroll-client -u "$adminUser" -p "$adminPassword" -un "$printClientComponentName"
    PrintWarningIfErrorAndContinue

    echo "unenrolling client component: $sharedEngineClientName..."
    "$scriptDir/NDI.SecClientConfig" unenroll-client -u "$adminUser" -p "$adminPassword" -un "$sharedEngineClientName"
    PrintWarningIfErrorAndContinue

    if [ "$unenrollDrcMode" == "unenroll_drc" ] ; then
        echo "unenrolling service component: $drcServicetName ..."
        "$scriptDir/NDI.SecClientConfig" unenroll-service -u "$adminUser" -p "$adminPassword" -on "$drcClientName" -s $drcServicetName
        PrintWarningIfErrorAndContinue

        echo "unenrolling client component: $drcClientName..."
        "$scriptDir/NDI.SecClientConfig" unenroll-client -u "$adminUser" -p "$adminPassword" -un "$drcClientName"
        eqCheckExitStatus
    else
        echo "skipping DRC unenrollment."
    fi
}

######################################

Usage()
{
    echo
    echo "Usage:"
    echo "   NDI.SecurityConfig.sh enroll __security_admin_user_name__ __security_admin_user_passsword__ __sfs_host_name__ __datacenter_name__ __print_groups_list__ [ enroll_drc ]"
    echo "       where:"
    echo "       __print_groups_list__ is semicolon separated list of user groups allowed to print on this Mac or 'eqeveryone' for everyone."
    echo "       If 'eqeveryone' is a real group - it will be used. Groups list should be enclosed in quotes i.e. \"group1;group2;group3\"."
    echo "or"
    echo "   NDI.SecurityConfig.sh unenroll __security_admin_user_name__ __security_admin_user_passsword__ [ unenroll_drc ]"
    echo
}

############ script body ############

iNumParams=$#
echo "iNumParams =" $iNumParams

scriptDir=$(dirname "$0")
configMode=$1
adminUser=$2
adminPassword=$3

printAdminComponentName=eqprintadmin
printClientComponentName=eqprintclient
drcClientName=DRE
drcServicetName=v1-eqdre
sharedEngineClientName=SE

echo "configMode =" $configMode
echo "adminUser =" $adminUser

if [ "$configMode" == "help" ] ; then
    Usage
    exit 0
fi

if [ "$configMode" == "" ] ; then
    Usage
    exit 1
fi

if [ "$configMode" == "enroll" ] ; then
    sfsHost=$4
    datacenterName=$5
    printGroupsList=$6
    enrollDrcMode=$7
    EnrollAllComponents
else
    if [ "$configMode" == "unenroll" ] ; then
        unenrollDrcMode=$4
        UnenrollAllComponents
    else
        echo "Error: invalid config mode"
        Usage
        exit 1
    fi
fi

echo
echo
echo "Security config succeeded."
echo

exit 0
